#include "memory_preallocated.h"

/* Initialise the memory pool with a mandatory 'limit' in pages. */

MemoryPreallocated::MemoryPreallocated(unsigned int limit, offset_t region_size)
: Memory(region_size), _limit(limit)
{
    while (limit > 0)
    {
        Region *r = region();

        if (r != NULL)
            _free.push_back(r);
        else
            break;

        limit--;
    }
}

/* Allocate a new region of the configured size rounded to the nearest page. */

Region *MemoryPreallocated::region()
{
    std::lock_guard<std::mutex> guard(_lock);

    std::list<Region *>::iterator it = _free.begin();

    if (it == _free.end())
        return NULL;

    Region *r = *it;

    _free.erase(it);
    _used.insert(r);

    return r;
}

/* Release the allocated 'region'. */

void MemoryPreallocated::release(Region *region)
{
    std::lock_guard<std::mutex> guard(_lock);

    std::set<Region *>::iterator it = _used.find(region);

    if (it != _used.end())
    {
        _used.erase(it);
        _free.push_back(region);
    }
}

// vim: tabstop=4 expandtab shiftwidth=4
